{-# LANGUAGE DeriveGeneric #-}
module Library.Types.Communication where

import Data.Aeson

import Data.UUID

import GHC.Generics

-- internal imports

import Library.Types.Player
import Library.Types.Map

data ServerMessage
  = ServerQuit               -- ^ Message notifying a client that the server is about to shut down.
  | AcceptClient             -- ^ Message notifying the client of being accepted and providing it's
                             --   newly registered 'UUID'
    { acClientUUID :: UUID
    }
  | ProvideInitialWizard     -- ^ Message containing a newly created player character
    { initWizard :: Wizard
    }
  | Ping                     -- ^ A server's heartbeat request
    { pingID :: UUID         -- ^ The random ID of the heartbeat request. NOT THE CLIENT'S 'UUID'!
    }
  | TickUpdate               -- ^ The Server's state update to the clients
    { tuMapSlice :: MapSlice -- ^ The player's field of vision
    , tuWizard   :: Wizard   -- ^ the player's updated character
    }
  deriving (Eq, Show, Generic)

instance FromJSON ServerMessage

instance ToJSON ServerMessage

data ClientMessagePayload
  = ClientQuit          -- ^ Message notifying the server of a client about to shut down
  | ClientRequestWizard -- ^ Message for requesting a new player character
  | ClientReady         -- ^ Message requesting the server to position the client's character on the
                        --   map
  | Pong                -- ^ A client's reply to a heartbeat request
    { pongId :: UUID    -- ^ The 'UUID' of the original heartbeat request
    }
  deriving (Eq, Show, Generic)

data ClientMessage
  = ClientMessage                        -- ^ A general client message to pass to the serve
    { cmClientID :: UUID                 -- ^ The client's 'UUID' for identification
    , cmPayload  :: ClientMessagePayload -- ^ The actual message payload
    }
  | IdRequest                            -- ^ A special message to the server for requesting a new
                                         --   'UUID' ad startup
  deriving (Eq, Show, Generic)

instance FromJSON ClientMessagePayload

instance ToJSON ClientMessagePayload

instance FromJSON ClientMessage

instance ToJSON ClientMessage
